# Generated by Django 2.0.4 on 2018-04-26 18:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("djnexmo", "0002_auto_20180426_1521")]

    operations = [
        migrations.AlterField(
            model_name="smsmessagepart",
            name="message_id",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterUniqueTogether(
            name="smsmessagepart", unique_together={("concat_ref", "concat_part")}
        ),
    ]
